@@ -1,4 +1,3 @@
-import warnings
 from itertools import combinations
 
 import numpy as np
@@ -119,14 +118,14 @@ def test_functions_inds_by_cutoffs(cluster_subspace):
 
 
 @pytest.mark.parametrize("orthonormal", [(True, False)])
-def test_site_bases(cluster_subspace, basis_name, orthonormal):
+def test_site_bases(cluster_subspace, basis_name, orthonormal, rng):
     subspace = cluster_subspace.copy()  # copy it to keep original state
     subspace.change_site_bases(basis_name, orthonormal=orthonormal)
     if orthonormal:
         assert subspace.basis_orthogonal
         assert subspace.basis_orthonormal
 
-    structure = gen_random_structure(subspace.structure)
+    structure = gen_random_structure(subspace.structure, rng=rng)
     if cluster_subspace.basis_type == subspace.basis_type and not orthonormal:
         npt.assert_array_almost_equal(
             subspace.corr_from_structure(structure),
@@ -165,22 +164,13 @@ def test_supercell_matrix_from_structure(cluster_subspace, rng):
     assert np.linalg.det(sc_matrix) == pytest.approx(8)
 
 
+@pytest.mark.xfail(raises=StructureMatchError)
 def test_refine_structure(cluster_subspace, rng):
     supercell = cluster_subspace.structure.copy()
     supercell.make_supercell(3)
-    structure = gen_random_structure(cluster_subspace.structure, size=3)
+    structure = gen_random_structure(cluster_subspace.structure, size=3, rng=rng)
     structure.apply_strain(rng.uniform(-0.01, 0.01, size=3))
-
-    #  TODO this sometimes fails because sc_matrix is not found!!!
-    try:
-        refined_structure = cluster_subspace.refine_structure(structure)
-    except StructureMatchError:
-        warnings.warn(
-            "StructureMatchError was raised when testing refined structures"
-            f" for testing structure {cluster_subspace.structure}",
-            RuntimeWarning,
-        )
-        return
+    refined_structure = cluster_subspace.refine_structure(structure)
 
     assert not np.allclose(  # check that distorted structure is not equivalent
         supercell.lattice.parameters, structure.lattice.parameters
@@ -216,12 +206,18 @@ def test_remove_orbits(cluster_subspace, rng):
         for i in range(len(cluster_subspace))
         if cluster_subspace.function_orbit_ids[i] not in ids_to_remove
     ]
-    structure = gen_random_structure(subspace.structure, size=2)
+    structure = gen_random_structure(subspace.structure, size=2, rng=rng)
     npt.assert_allclose(
         cluster_subspace.corr_from_structure(structure)[corr_inds],
         subspace.corr_from_structure(structure),
     )
 
+    # remove all orbits of a certain size and make sure key is removed
+    size = rng.choice(list(subspace.orbits_by_size.keys()))
+    ids_to_remove = [o.id for o in subspace.orbits_by_size[size]]
+    subspace.remove_orbits(ids_to_remove)
+    assert size not in subspace.orbits_by_size.keys()
+
     with pytest.raises(ValueError):
         subspace.remove_orbits([-1])
     with pytest.raises(ValueError):
@@ -242,7 +238,7 @@ def test_remove_corr_functions(cluster_subspace, rng):
     )
 
     corr_inds = [i for i in range(len(cluster_subspace)) if i not in ids_to_remove]
-    structure = gen_random_structure(subspace.structure, size=2)
+    structure = gen_random_structure(subspace.structure, size=2, rng=rng)
     npt.assert_allclose(
         cluster_subspace.corr_from_structure(structure)[corr_inds],
         subspace.corr_from_structure(structure),
@@ -253,12 +249,12 @@ def test_remove_corr_functions(cluster_subspace, rng):
         subspace.remove_corr_functions(ids)
 
 
-def test_orbit_mappings(cluster_subspace, supercell_matrix):
+@pytest.mark.xfail(raises=StructureMatchError)
+def test_orbit_mappings(cluster_subspace, supercell_matrix, rng):
     # check that all supercell_structure index groups map to the correct
     # primitive cell sites, and check that max distance under supercell
     # structure pbc is less than the max distance without pbc
 
-    # sc_matrix = np.array([[2, 0, 0], [0, 2, 0], [0, 1, 1]])
     supercell_struct = cluster_subspace.structure.copy()
     supercell_struct.make_supercell(supercell_matrix)
     fcoords = np.array(supercell_struct.frac_coords)
@@ -282,16 +278,109 @@ def test_orbit_mappings(cluster_subspace, supercell_matrix):
             assert found
 
     # check that the matrix was cached
-    m_hash = tuple(sorted(tuple(s) for s in supercell_matrix))
+    m_hash = tuple(sorted(tuple(s.tolist()) for s in supercell_matrix))
     assert cluster_subspace._supercell_orb_inds[
         m_hash
     ] is cluster_subspace.supercell_orbit_mappings(supercell_matrix)
 
+    # Test that symmetrically equivalent matrices really produce the
+    # same correlation vector for the same occupancy.
+    structures = [
+        gen_random_structure(cluster_subspace.structure, size=supercell_matrix, rng=rng)
+        for _ in range(10)
+    ]
+    matrix2 = np.array([[0, 1, 0], [0, 0, 1], [1, 0, 0]]) @ np.array(
+        supercell_matrix, dtype=int
+    )
+    matrix3 = np.array([[0, 0, 1], [-1, 0, 0], [0, 1, 0]]) @ np.array(
+        supercell_matrix, dtype=int
+    )
+
+    # Because of different site_mappings, when you change super-cell,
+    # you must re-generate occupancy strings altogether even if your
+    # super-cell matrices are symmetrically equivalent.
+    occus = [
+        cluster_subspace.occupancy_from_structure(
+            structure, scmatrix=supercell_matrix, encode=True
+        )
+        for structure in structures
+    ]
+    occus2 = [
+        cluster_subspace.occupancy_from_structure(
+            structure, scmatrix=matrix2, encode=True
+        )
+        for structure in structures
+    ]
+    occus3 = [
+        cluster_subspace.occupancy_from_structure(
+            structure, scmatrix=matrix3, encode=True
+        )
+        for structure in structures
+    ]
+
+    corrs = np.array(
+        [
+            corr_from_occupancy(
+                occu,
+                cluster_subspace.num_corr_functions,
+                cluster_subspace.gen_orbit_list(supercell_matrix),
+            )
+            for occu in occus
+        ]
+    )
+
+    corrs2 = np.array(
+        [
+            corr_from_occupancy(
+                occu,
+                cluster_subspace.num_corr_functions,
+                cluster_subspace.gen_orbit_list(matrix2),
+            )
+            for occu in occus2
+        ]
+    )
+
+    corrs3 = np.array(
+        [
+            corr_from_occupancy(
+                occu,
+                cluster_subspace.num_corr_functions,
+                cluster_subspace.gen_orbit_list(matrix3),
+            )
+            for occu in occus3
+        ]
+    )
+
+    # Symmetrically equivalent matrices should give the same correlation
+    # function for the same structure, when the orbit indices mappings
+    # are re-generated.
+    npt.assert_array_almost_equal(corrs, corrs2)
+    npt.assert_array_almost_equal(corrs, corrs3)
+
+    # Symmetrically equivalent matrices should give the same correlation
+    # vectors on the same structure, when using the default orbit mapping.
+    cluster_subspace._supercell_orb_inds = {}
+    for structure, expected in zip(structures, corrs):
+        predicted = cluster_subspace.corr_from_structure(
+            structure, scmatrix=supercell_matrix
+        )
+        npt.assert_array_almost_equal(predicted, expected)
+
+        predicted = cluster_subspace.corr_from_structure(structure, scmatrix=matrix2)
+        npt.assert_array_almost_equal(predicted, expected)
+
+        predicted = cluster_subspace.corr_from_structure(structure, scmatrix=matrix3)
+        npt.assert_array_almost_equal(predicted, expected)
+
+        predicted = cluster_subspace.corr_from_structure(structure)
+        npt.assert_array_almost_equal(predicted, expected)
+
 
 def test_get_aliased_orbits(cluster_subspace, supercell_matrix):
     # Verify that
     # 1) site mappings for aliased orbits are indeed an identical set,
     # 2) each orbit is counted only once,
+    # 3) New implementation is the same with the old one.
     aliased_orbs = cluster_subspace.get_aliased_orbits(supercell_matrix)
     sc_orbit_maps = cluster_subspace.supercell_orbit_mappings(supercell_matrix)
     accounted_orbs = []
@@ -308,10 +397,33 @@ def test_get_aliased_orbits(cluster_subspace, supercell_matrix):
                 )
                 assert set(orbit_map_i) == set(orbit_map_j)
 
+    aliased_orbits_std = []
+    for orb_i, orb_map_i in enumerate(sc_orbit_maps):
+        orb_i_id = orb_i + 1
+        aliased = False
+        orbit_i_aliased = [orb_i_id]
+        sorted_orb_map_i = {tuple(sorted(c_map)) for c_map in orb_map_i}
 
-def test_periodicity_and_symmetry(cluster_subspace, supercell_matrix):
+        for orb_j, orb_map_j in enumerate(sc_orbit_maps):
+            if orb_i == orb_j:
+                continue
+            orb_j_id = orb_j + 1
+            sorted_orb_map_j = {tuple(sorted(c_map)) for c_map in orb_map_j}
 
-    structure = gen_random_structure(cluster_subspace.structure, size=2)
+            if sorted_orb_map_i == sorted_orb_map_j:
+                aliased = True
+                orbit_i_aliased.append(orb_j_id)
+
+        orbit_i_aliased = tuple(sorted(orbit_i_aliased))
+        if aliased:
+            aliased_orbits_std.append(orbit_i_aliased)
+
+    aliased_orbits_std = sorted(list(set(aliased_orbits_std)), key=lambda x: x[0])
+    assert aliased_orbits_std == aliased_orbs
+
+
+def test_periodicity_and_symmetry(cluster_subspace, supercell_matrix, rng):
+    structure = gen_random_structure(cluster_subspace.structure, size=2, rng=rng)
     larger_structure = structure.copy()
     larger_structure.make_supercell(supercell_matrix)
 
@@ -381,7 +493,7 @@ def test_msonable(cluster_subspace_ewald, rng):
 
     for _ in range(2):
         size = rng.integers(1, 4)
-        s = gen_random_structure(cluster_subspace_ewald.structure, size=size)
+        s = gen_random_structure(cluster_subspace_ewald.structure, size=size, rng=rng)
         _ = cluster_subspace_ewald.corr_from_structure(s)
 
     assert_msonable(cluster_subspace_ewald)
@@ -626,6 +738,67 @@ def test_periodicity(single_subspace):
     npt.assert_allclose(a, b)
 
 
+def test_site_basis_rotation(cluster_subspace, rng):
+    cs = cluster_subspace.copy()
+    if not cs.basis_orthogonal:
+        with pytest.raises(RuntimeError):
+            cs.rotate_site_basis(1, np.pi / 4)
+        cs.change_site_bases("sinusoid", orthonormal=True)
+
+    with pytest.raises(ValueError):
+        cs.rotate_site_basis(len(cs.orbits_by_size[1]) + 2, np.pi)
+
+    cs1 = cs.copy()
+    print(cs.site_rotation_matrix)
+    cs1.rotate_site_basis(1, np.pi / 4)
+    print(cs1.site_rotation_matrix)
+    for i in range(5):
+        structure = gen_random_structure(cs.structure, rng=rng)
+        for j in range(5):
+            coefs1 = 10 * np.random.random(len(cs))
+            coefs = coefs1.copy()
+            coefs = cs1.site_rotation_matrix.T @ coefs
+            eci = coefs / cs.function_ordering_multiplicities
+            eci1 = coefs1 / cs1.function_ordering_multiplicities
+            norms = np.array(
+                [
+                    np.sum(
+                        cs.function_ordering_multiplicities[
+                            np.array(cs.function_orbit_ids) == i
+                        ]
+                        * eci[np.array(cs.function_orbit_ids) == i] ** 2
+                    )
+                    for i in range(len(cs.orbits) + 1)
+                ]
+            )
+            norms1 = np.array(
+                [
+                    np.sum(
+                        cs1.function_ordering_multiplicities[
+                            np.array(cs.function_orbit_ids) == i
+                        ]
+                        * eci1[np.array(cs1.function_orbit_ids) == i] ** 2
+                    )
+                    for i in range(len(cs.orbits) + 1)
+                ]
+            )
+            # test ECI invariance
+            assert np.allclose(norms, norms1)
+            # test correlation vector invariance
+            assert np.allclose(
+                cs1.site_rotation_matrix @ cs.corr_from_structure(structure),
+                cs1.corr_from_structure(structure),
+            )
+            # test values are the same
+            assert np.isclose(
+                np.dot(cs.corr_from_structure(structure), coefs),
+                np.dot(cs1.corr_from_structure(structure), coefs1),
+            )
+            # test rotation matrix is invertible and abs(det) = 1
+            # binary site rotations may lead to det = -1 ?
+            assert np.isclose(abs(np.linalg.det(cs1.site_rotation_matrix)), 1)
+
+
 def _encode_occu(occu, bits):
     return np.array([bit.index(sp) for sp, bit in zip(occu, bits)])
 
